# Generated by Django 2.2.24 on 2021-11-29 10:44

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("mozilla_django_oidc_db", "0005_openidconnectconfig_sync_groups_glob_pattern"),
    ]

    operations = [
        migrations.AddField(
            model_name="openidconnectconfig",
            name="username_claim",
            field=models.CharField(
                default="sub",
                help_text="The name of the OIDC claim that is used as the username",
                max_length=50,
                verbose_name="username claim",
            ),
        ),
    ]
