# Generated by Django 3.2.23 on 2024-02-05 16:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mozilla_django_oidc_db", "0014_alter_openidconnectconfig_groups_claim"),
    ]

    operations = [
        migrations.AddField(
            model_name="openidconnectconfig",
            name="oidc_token_use_basic_auth",
            field=models.BooleanField(
                default=False,
                help_text="If enabled, the client ID and secret are sent in the HTTP Basic auth header when obtaining the access token. Otherwise, they are sent in the request body.",
                verbose_name="Use Basic auth for token endpoint",
            ),
        ),
    ]
