# Generated by Django 3.2.18 on 2023-12-21 14:07

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            "mozilla_django_oidc_db",
            "0011_alter_openidconnectconfig_userinfo_claims_source",
        ),
    ]

    operations = [
        migrations.AlterField(
            model_name="openidconnectconfig",
            name="sync_groups",
            field=models.BooleanField(
                default=True,
                help_text="If checked, local user groups will be created for group names present in the groups claim, if they do not exist yet locally.",
                verbose_name="Create local user groups if they do not exist yet",
            ),
        ),
    ]
